@@ -7,7 +7,6 @@
 
 import os
 import sys
-import stat
 import imp
 import urlparse
 
@@ -42,41 +41,63 @@ class ResponseMixin:
 
     def get_header(self, name):
         try:
-            seqnum, value = self.__headers[name]
-            return value
+            seqnum, name, value = self.__headers[name.lower()]
         except KeyError:
             pass
+        else:
+            return value
 
     def set_header(self, name, value):
+        """
+        Sets a header to the given value, overriding any
+        previous value.
+        """
         if self.__sent_headers:
             raise ApplicationError('headers have already been sent')
+        name_lower = name.lower()
         try:
-            seqnum, old_value = self.__headers[name]
-            self.__headers[name] = (seqnum, value)
-            return
+            seqnum, name, old_value = self.__headers[name_lower]
         except KeyError:
-            pass
-        self.__headers[name] = (self.__seqnum, value)
-        self.__seqnum = self.__seqnum + 1
+            self.__headers[name_lower] = (self.__seqnum, name, [value])
+            self.__seqnum += 1
+        else:
+            self.__headers[name_lower] = (seqnum, name, [value])
+
+    def add_header(self, name, value):
+        """
+        Sets the value of the name header to value in
+        the internal store, appending the new header
+        immediately after any existing headers of the
+        same name.
+        """
+        if self.__sent_headers:
+            raise ApplicationError('headers have already been sent')
+        name_lower = name.lower()
+        try:
+            seqnum, name, old_value = self.__headers[name_lower]
+        except KeyError:
+            self.__headers[name_lower] = (self.__seqnum, name, [value])
+            self.__seqnum += self.__seqnum
+        else:
+            old_value.append(value)
+            self.__headers[name_lower] = (seqnum, name, old_value)
 
     def del_header(self, name):
         if self.__sent_headers:
             raise ApplicationError('headers have already been sent')
         try:
-            del self.__headers[name]
+            del self.__headers[name.lower()]
         except KeyError:
             pass
 
     def write_headers(self):
         if self.__sent_headers:
             raise ApplicationError('headers have already been sent')
-        headers = []
-        for name, seqnum_value in self.__headers.items():
-            seqnum, value = seqnum_value
-            headers.append((seqnum, name, value))
+        headers = self.__headers.values()
         headers.sort()
-        for seqnum, name, value in headers:
-            self.request.write_header(name, value)
+        for seqnum, name, values in headers:
+            for value in values:
+                self.request.write_header(name, value)
         self.request.end_headers()
         self.__sent_headers = 1
 
@@ -86,9 +107,10 @@ class ResponseMixin:
         self.request.write_content(data)
 
     def send_redirect(self, loc):
-        cookie = self.get_header('Set-Cookie')
-        if cookie:
-            self.request.write_header('Set-Cookie', cookie)
+        cookies = self.get_header('Set-Cookie')
+        if cookies:
+            for cookie in cookies:
+                self.request.write_header('Set-Cookie', cookie)
         return self.request.redirect(loc)
 
 
@@ -103,6 +125,7 @@ class AppContext(NamespaceMixin, ResponseMixin, ExecuteMixin):
         self.locals.__page__ = None
         self.locals.__pages__ = []
         self.app = app
+        self.__current_uri = None
 
     # ----------------------------------------------------
     # Serve all requests for resources via the application
@@ -190,34 +213,46 @@ class AppContext(NamespaceMixin, ResponseMixin, ExecuteMixin):
             if self.request.field_value(name): 
                 return 1
         except KeyError:
-            pass
+            try:
+                if self.request.field_value(name + '.x'):
+                    return 1
+            except KeyError:
+                pass
         return 0
 
     def base_url(self):
         return self.app.base_url()
 
+    def parsed_request_uri(self):
+        if self.__current_uri is None:
+            self.__current_uri = urlparse.urlparse(self.request.get_uri())[:3]
+        return self.__current_uri
+
     def current_url(self):
-        uri = self.request.get_uri()
-        scheme, netloc, path = urlparse.urlparse(uri)[:3]
-        return path
+        return self.parsed_request_uri()[2]
+
+    def absolute_base_url(self):
+        this_path = self.current_url()
+        base_path = urlparse.urlparse(self.base_url())[2]
+        pos = this_path.find(base_path)
+        if pos < 0:
+            raise ApplicationError('base_url "%s" not found in request url "%s"' % (base_path, this_path))
+        return this_path[:pos + len(base_path)]
 
     def redirect_url(self, loc):
-        this_path = urlparse.urlparse(self.request.get_uri())[2]
-        base_path = urlparse.urlparse(self.app.base_url())[2]
-        pos = this_path.find(base_path)
-        new_base = this_path[:pos + len(base_path)]
+        scheme, netloc, path = urlparse.urlparse(loc)[:3]
+        if scheme or netloc:
+            return loc                  # Already absolute
+        scheme, netloc, path = self.parsed_request_uri()
+        if loc.startswith('/'):
+            return urlparse.urlunparse((scheme, netloc, loc, '', '', ''))
+        new_base = self.absolute_base_url()
         if new_base[-1] != '/':
             new_base = new_base + '/'
-        return urlparse.urlunparse(('http', self.request.get_servername(),
-                                    new_base + loc, '', '', ''))
+        return urlparse.urlunparse((scheme, netloc, new_base + loc, '', '', ''))
 
     def redirect(self, loc):
-        scheme, netloc, path = urlparse.urlparse(loc)[:3]
-        if scheme or netloc or (path and path[0] == '/'):
-            raise Redirect(loc)
-        # Redirect from base url - turn into full URL redirect
-        new_loc = self.redirect_url(loc)
-        raise Redirect(new_loc)
+        raise Redirect(self.redirect_url(loc))
 
 
 # Basic application processing sequence is implemented in a class
@@ -253,7 +288,7 @@ class Application(ResourceMixin):
             return ctx.send_redirect(e.loc)
         except:
             self.handle_exception(ctx, req)
-        return req.status()
+        return req.return_code()
 
     def format_exception(self):
         import traceback
@@ -273,6 +308,7 @@ class Application(ResourceMixin):
 
     def handle_exception(self, ctx, req):
         pyexc, htmlexc = map(tags.escape, self.format_exception())
+        req.set_status(HTTP_INTERNAL_SERVER_ERROR)
         req.write_header('Content-Type', 'text/html')
         req.end_headers()
         try:
@@ -343,10 +379,11 @@ class PageModuleMixin:
     '''Caching module loader
     '''
 
+    mod_holder_name = '__alpage__'
+
     def __init__(self, base_dir, start_page):
         self.__base_dir = base_dir
         self.__start_page = start_page
-        self.__cache = {}
 
     def module_path(self):
         return self.__base_dir
@@ -362,35 +399,52 @@ class PageModuleMixin:
             ctx.set_page(name)
         self.load_page_module(ctx, name)
 
+    def is_page_module(self, name):
+        return name.startswith(self.mod_holder_name)
+
     def load_page_module(self, ctx, name):
-        if self.__base_dir == '.':
-            path = name
+        mod_path = name.replace('/', '.').split('.')
+        if mod_path[0] != self.mod_holder_name:
+            mod_path.insert(0, self.mod_holder_name)
+        # Create module path out of dummy modules, if needed
+        for i in range(len(mod_path) - 1, 0, -1):
+            mod_name = '.'.join(mod_path[:i])
+            try:
+                sys.modules[mod_name]
+                break
+            except KeyError:
+                sys.modules[mod_name] = imp.new_module(mod_name)
+        # Now see if it's already loaded
+        mod_name = '.'.join(mod_path)
+        try:
+            module = sys.modules[mod_name]
+        except KeyError:
+            # Nope
+            pass
         else:
-            path = os.path.join(self.__base_dir, name)
-        page = self.__cache.get(path)
-        if page:
-            mtime = os.stat(page.__file__)[stat.ST_MTIME]
-            if mtime > page.__mtime__:
-                reload(page)
-                page.__mtime__ = mtime
-        if not page:
-            dirname, name = os.path.split(path)
-            file, filepath, desc = imp.find_module(name, [dirname])
-            page = sys.modules.get(name)
-            if not page or not page.__file__.startswith(filepath):
-                try:
-                    page = imp.load_module(name, file, filepath, desc)
-                finally:
-                    if file:
-                        file.close()
-            if not (hasattr(page, 'page_enter') or 
-                    hasattr(page, 'page_leave') or 
-                    hasattr(page, 'page_process') or 
-                    hasattr(page, 'page_display')):
-                raise ApplicationError('module "%s" does not define one of page_enter, page_leave, page_process or page_display' % (name))
-            page.__mtime__ = os.stat(page.__file__)[stat.ST_MTIME]
-            self.__cache[path] = page
-        ctx.page = page
+            # The name exists, but is it a dummy or the real thing?
+            if hasattr(module, '__file__'):
+                # Real? Then return it
+                ctx.page = module
+                return module
+            # Dummy? but we want the real thing... so continue
+        mod_dir = os.path.join(self.__base_dir, *mod_path[1:-1])
+        try:
+            f, filepath, desc = imp.find_module(mod_path[-1], [mod_dir])
+        except ImportError, e:
+            raise ApplicationError('%s (in %s)' % (e, mod_dir))
+        try:
+            module = imp.load_module(mod_name, f, filepath, desc)
+        finally:
+            if f:
+                f.close()
+        if not (hasattr(module, 'page_enter') or
+                hasattr(module, 'page_leave') or
+                hasattr(module, 'page_process') or
+                hasattr(module, 'page_display')):
+            raise ApplicationError('module "%s" does not define one of page_enter, page_leave, page_process or page_display' % (name))
+        sys.modules[mod_name] = ctx.page = module
+        return module
 
     def page_enter(self, ctx, args):
         if hasattr(ctx.page, 'page_enter'):
@@ -420,8 +474,8 @@ class PageObjectMixin:
         self.__start_page = start_page
         self.__page_objects = {}
 
-    def module_path(self):
-        pass
+    def is_page_module(self, name):
+        return False
 
     def start_page(self):
         return self.__start_page
