@@ -10,6 +10,7 @@ import sys
 import stat
 import re
 import cPickle
+import __builtin__
 
 try:
     import zlib
@@ -259,7 +260,7 @@ class NameRecorderMixin:
         text = base64.encodestring(text)
         self.write_content('<input type="hidden" name="__albform__" value="')
         self.write_content(text)
-        self.write_content('">\n')
+        self.write_content('" />\n')
         self.__elem_names = {}
         return self.__need_multipart_enc
 
@@ -564,9 +565,13 @@ class SessionBase:
         self.clear_locals()
 
     def decode_session(self, text):
-        module_path = self.app.module_path()
-        if module_path:
-            sys.path.insert(0, module_path)
+        def imp_hook(name, globals, locals, fromlist):
+            if self.app.is_page_module(name):
+                return self.app.load_page_module(self, name)
+            else:
+                return real_imp(name, globals, locals, fromlist)
+
+        real_imp, __builtin__.__import__ = __builtin__.__import__, imp_hook
         try:
             try:
                 vars = cPickle.loads(text)
@@ -577,8 +582,7 @@ class SessionBase:
             for name in vars.keys():
                 self.__vars[name] = 1
         finally:
-            if module_path:
-                sys.path.remove(module_path)
+            __builtin__.__import__ = real_imp
 
     def encode_session(self):
         vars = {}
@@ -631,7 +635,7 @@ class HiddenFieldSessionMixin(SessionBase):
             text = self.encode_session()
             self.write_content('<input type="hidden" name="__albstate__" value="')
             self.write_content(text)
-            self.write_content('">\n')
+            self.write_content('" />\n')
 
 
 # ------------------------------------------------------------------
