@@ -21,11 +21,14 @@ class Request(cgiapp.Request):
     def get_uri(self):
         return self.__fcgi.env.get('REQUEST_URI')
                                                                      
+    def get_method(self):
+        return self.__fcgi.env.get('REQUEST_METHOD')
+                                                                     
     def get_path_info(self):
         return self.__fcgi.env.get('PATH_INFO')
 
     def get_servername(self):
-        return self.__fcgi.env.get('SERVER_NAME')
+        return self.__fcgi.env.get('HTTP_HOST')
 
     def get_header(self, name):
         env_name = 'HTTP_' + name.upper().replace('-', '_')
