@@ -62,6 +62,15 @@ class EvalAttribMixin:
             self.raise_error('"name" attribute is null')
         return name
 
+    def get_name_and_value(self, ctx):
+        name = self.get_name(ctx)
+        if self.has_attrib('expr'):
+            return name, self.eval_attrib(ctx, 'expr')
+        value = self.get_attrib('value')
+        if value is not None:
+            return name, value
+        else:
+            return name, ctx.get_value(name)
 
 # ------------------------------------------------------------------
 
@@ -74,6 +83,12 @@ class Input(EmptyTag, EvalAttribMixin):
 
     name = 'al-input'
 
+    naming_attrs = ('name', 'nameexpr', 'alias',
+                    'prevpage', 'nextpage', 
+                    'treefold', 'treeselect', 'treeellipsis')
+
+    all_attrs = ('expr', 'valueexpr', 'list', 'noescape', 'node') + naming_attrs
+
     def __init__(self, ctx, filename, line_num, attribs):
         EmptyTag.__init__(self, ctx, filename, line_num, attribs)
         self.__itype = self.get_attrib('type', 'text').lower()
@@ -81,17 +96,10 @@ class Input(EmptyTag, EvalAttribMixin):
             self.unbound_to_html = self.type_dict[self.__itype]
         except KeyError:
             self.raise_error('unrecognised <INPUT TYPE="%s">' % self.__itype)
-        self.assert_has_attribs('name', 'nameexpr', 'alias',
-                                'prevpage', 'nextpage', 
-                                'treefold', 'treeselect', 'treeellipsis')
+        self.assert_has_attribs(*self.naming_attrs)
 
     def write_attribs_except(self, ctx, *attribs):
-        EmptyTag.write_attribs_except(self, ctx,
-               'name', 'nameexpr', 'alias',
-               'expr', 'valueexpr',
-               'list', 'noescape',
-               'nextpage', 'prevpage',
-               'treefold', 'treeselect', 'treeellipsis', 'node',  *attribs)
+        EmptyTag.write_attribs_except(self, ctx, *(self.all_attrs + attribs))
 
     def get_name(self, ctx):
         for name in ('prevpage', 'nextpage'):
@@ -109,16 +117,6 @@ class Input(EmptyTag, EvalAttribMixin):
                 return '%s,%s,%s' % (name, iter_name, node.albatross_alias())
         return EvalAttribMixin.get_name(self, ctx)
 
-    def get_name_and_value(self, ctx):
-        name = self.get_name(ctx)
-        if self.has_attrib('expr'):
-            return name, self.eval_attrib(ctx, 'expr')
-        value = self.get_attrib('value')
-        if value is not None:
-            return name, value
-        else:
-            return name, ctx.get_value(name)
-
     def get_name_value_checked(self, ctx, default = None):
         # Name
         name = self.get_name(ctx)
@@ -139,7 +137,7 @@ class Input(EmptyTag, EvalAttribMixin):
     def to_html(self, ctx):
         ctx.write_content('<input')
         self.unbound_to_html(self, ctx)
-        ctx.write_content('>')
+        ctx.write_content(' />')
 
     def generic_to_html(self, ctx):
         self.write_attribs_except(ctx, 'value')
@@ -265,7 +263,7 @@ class Img(EmptyTag, EvalAttribMixin):
             ctx.write_content(' src="%s"' % src)
         else:
             ctx.write_content(' src="%s"' % escape(src))
-        ctx.write_content('>')
+        ctx.write_content(' />')
 
 
 # Using the albatross <FORM> tag allows Albatross to record the
@@ -353,12 +351,11 @@ class Select(EnclosingTag, EvalAttribMixin):
         self.options = []
 
     def to_html(self, ctx):
-        name = self.get_name(ctx)
+        name, value = self.get_name_and_value(ctx)
         ctx.write_content('<select')
         self.write_attribs_except(ctx, 'optionexpr', 'alias', 'name',
                                   'nameexpr', 'noescape')
         ctx.write_content(' name="%s">' % name)
-        value = ctx.get_value(name)
         if type(value) in (type([]), type(())):
             value = map(str, value)
         else:
