@@ -10,8 +10,11 @@ class ListPage:
         ctx.run_template('paginate.html')
 
 
-app = SimpleSessionApp('paginate.py', '.', 'start', 'paginate',
-                       '-=-secret-=-')
+app = SimpleSessionApp(base_url='paginate.py',
+                       template_path='.',
+                       start_page='start',
+                       secret='-=-secret-=-',
+                       session_appid='paginate')
 app.register_page('start', ListPage())
 
 
