@@ -5,7 +5,7 @@ from albatross import SimpleContext, TemplateLoadError
 script_name = os.environ['SCRIPT_NAME']
 request_uri = os.environ['REQUEST_URI']
 page = request_uri[len(script_name) + 1:]
-if not page:
+if not page or os.path.dirname(page):
     page = 'main.html'
 
 ctx = SimpleContext('templ')
